@@ -1,8 +1,9 @@
 import time
 from libmproxy.script import concurrent
 
-@concurrent
+
+@concurrent  # Remove this and see what happens
 def request(context, flow):
     print "handle request: %s%s" % (flow.request.host, flow.request.path)
     time.sleep(5)
-    print "start  request: %s%s" % (flow.request.host, flow.request.path)
+    print "start  request: %s%s" % (flow.request.host, flow.request.path)
\ No newline at end of file
