@@ -31,8 +31,10 @@
         ~c CODE     Response code.
         rex         Equivalent to ~u rex
 """
+from __future__ import absolute_import
 import re, sys
-import contrib.pyparsing as pp
+from .contrib import pyparsing as pp
+from .protocol.http import decoded
 
 
 class _Token:
@@ -164,10 +166,14 @@ class FBod(_Rex):
     code = "b"
     help = "Body"
     def __call__(self, f):
-        if f.request.content and re.search(self.expr, f.request.content):
-            return True
-        elif f.response and f.response.content and re.search(self.expr, f.response.content):
-            return True
+        if f.request and f.request.content:
+            with decoded(f.request):
+                if re.search(self.expr, f.request.content):
+                    return True
+        if f.response and f.response.content:
+            with decoded(f.response):
+                if re.search(self.expr, f.response.content):
+                    return True
         return False
 
 
@@ -175,16 +181,20 @@ class FBodRequest(_Rex):
     code = "bq"
     help = "Request body"
     def __call__(self, f):
-        if f.request.content and re.search(self.expr, f.request.content):
-            return True
+        if f.request and f.request.content:
+            with decoded(f.request):
+                if re.search(self.expr, f.request.content):
+                    return True
 
 
 class FBodResponse(_Rex):
     code = "bs"
     help = "Response body"
     def __call__(self, f):
-        if f.response and f.response.content and re.search(self.expr, f.response.content):
-            return True
+        if f.response and f.response.content:
+            with decoded(f.response):
+                if re.search(self.expr, f.response.content):
+                    return True
 
 
 class FMethod(_Rex):
@@ -212,7 +222,7 @@ class FUrl(_Rex):
         return klass(*toks)
 
     def __call__(self, f):
-        return re.search(self.expr, f.request.get_url())
+        return re.search(self.expr, f.request.url)
 
 
 class _Int(_Action):
@@ -333,9 +343,34 @@ bnf = _make()
 
 def parse(s):
     try:
-        return bnf.parseString(s, parseAll=True)[0]
+        filt = bnf.parseString(s, parseAll=True)[0]
+        filt.pattern = s
+        return filt
     except pp.ParseException:
         return None
     except ValueError:
         return None
 
+
+help = []
+for i in filt_unary:
+    help.append(
+        ("~%s"%i.code, i.help)
+    )
+for i in filt_rex:
+    help.append(
+        ("~%s regex"%i.code, i.help)
+    )
+for i in filt_int:
+    help.append(
+        ("~%s int"%i.code, i.help)
+    )
+help.sort()
+help.extend(
+    [
+        ("!", "unary not"),
+        ("&", "and"),
+        ("|", "or"),
+        ("(...)", "grouping"),
+    ]
+)
\ No newline at end of file
