@@ -1,7 +1,7 @@
+from __future__ import absolute_import
 import os, datetime, urllib, re
 import time, functools, cgi
 import json
-from netlib import http
 
 def timestamp():
     """
@@ -64,12 +64,6 @@ def urlencode(s):
     return urllib.urlencode(s, False)
 
 
-def del_all(dict, keys):
-    for key in keys:
-        if key in dict:
-            del dict[key]
-
-
 def pretty_size(size):
     suffixes = [
         ("B",   2**10),
@@ -143,14 +137,6 @@ class LRUCache:
                 return ret
         return wrap
 
-
-def parse_proxy_spec(url):
-    p = http.parse_url(url)
-    if not p or not p[1]:
-        return None
-    return p[:3]
-
-
 def parse_content_type(c):
     """
         A simple parser for content-type values. Returns a (type, subtype,
