@@ -7,14 +7,18 @@ def serverconnect(ctx, cc):
     ctx.log("XSERVERCONNECT")
     log.append("serverconnect")
 
-def request(ctx, r):
+def request(ctx, f):
     ctx.log("XREQUEST")
     log.append("request")
 
-def response(ctx, r):
+def response(ctx, f):
     ctx.log("XRESPONSE")
     log.append("response")
 
+def responseheaders(ctx, f):
+    ctx.log("XRESPONSEHEADERS")
+    log.append("responseheaders")
+
 def clientdisconnect(ctx, cc):
     ctx.log("XCLIENTDISCONNECT")
     log.append("clientdisconnect")
