@@ -9,7 +9,7 @@ import tutils
 
 
 def test_format_flow():
-    f = tutils.tflow_full()
+    f = tutils.tflow(resp=True)
     assert common.format_flow(f, True)
     assert common.format_flow(f, True, hostheader=True)
     assert common.format_flow(f, True, extended=True)
