@@ -29,11 +29,11 @@ class TestScript:
         s = flow.State()
         fm = flow.FlowMaster(None, s)
         fm.load_script(tutils.test_data.path("scripts/duplicate_flow.py"))
-        r = tutils.treq()
-        fm.handle_request(r)
+        f = tutils.tflow()
+        fm.handle_request(f)
         assert fm.state.flow_count() == 2
-        assert not fm.state.view[0].request.is_replay()
-        assert fm.state.view[1].request.is_replay()
+        assert not fm.state.view[0].request.is_replay
+        assert fm.state.view[1].request.is_replay
 
     def test_err(self):
         s = flow.State()
@@ -65,37 +65,45 @@ class TestScript:
         fm.load_script(tutils.test_data.path("scripts/concurrent_decorator.py"))
 
         with mock.patch("libmproxy.controller.DummyReply.__call__") as m:
-            r1, r2 = tutils.treq(), tutils.treq()
+            f1, f2 = tutils.tflow(), tutils.tflow()
             t_start = time.time()
-            fm.handle_request(r1)
-            r1.reply()
-            fm.handle_request(r2)
-            r2.reply()
+            fm.handle_request(f1)
+            f1.reply()
+            fm.handle_request(f2)
+            f2.reply()
 
             # Two instantiations
-            assert m.call_count == 2
+            assert m.call_count == 0  # No calls yet.
             assert (time.time() - t_start) < 0.09
-            time.sleep(0.2)
-            # Plus two invocations
-            assert m.call_count == 4
 
     def test_concurrent2(self):
         s = flow.State()
         fm = flow.FlowMaster(None, s)
         s = script.Script(tutils.test_data.path("scripts/concurrent_decorator.py"), fm)
         s.load()
-        f = tutils.tflow_full()
-        f.error = tutils.terr(f.request)
-        f.reply = f.request.reply
+        m = mock.Mock()
 
-        with mock.patch("libmproxy.controller.DummyReply.__call__") as m:
-            s.run("clientconnect", f)
-            s.run("serverconnect", f)
-            s.run("response", f)
-            s.run("error", f)
-            s.run("clientdisconnect", f)
-            time.sleep(0.1)
-            assert m.call_count == 5
+        class Dummy:
+            def __init__(self):
+                self.response = self
+                self.error = self
+                self.reply = m
+
+        t_start = time.time()
+
+        for hook in ("clientconnect",
+                     "serverconnect",
+                     "response",
+                     "error",
+                     "clientconnect"):
+            d = Dummy()
+            assert s.run(hook, d)[0]
+            d.reply()
+        while (time.time() - t_start) < 20 and m.call_count <= 5:
+            if m.call_count == 5:
+                return
+            time.sleep(0.001)
+        assert False
 
     def test_concurrent_err(self):
         s = flow.State()
